@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  file console.h
- *  Copyright (C) 1998--2002  Guido Masarotto and Brian Ripley
+ *  Copyright (C) 1998--2003  Guido Masarotto and Brian Ripley
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,8 @@ typedef window dataeditor;
 
 void
 setconsoleoptions(char *fnname,int fnsty, int fnpoints,
-		  int rows, int cols, rgb nfg, rgb nufg, rgb nbg, rgb high,
+		  int rows, int cols, int consx, int consy,
+		  rgb nfg, rgb nufg, rgb nbg, rgb high,
 		  int pgr, int pgc, int multiplewindows, int widthonresize,
 		  int bufbytes, int buflines);
 pager newpager(char *title, char *filename, char *header, int unlinkonexit);
@@ -52,3 +53,4 @@ extern int setWidthOnResize;
 int consolecols(console c);
 void pagerbclose(control m);
 
+int graphicsx, graphicsy;
