@@ -5,7 +5,7 @@ function(data = NA, dim = length(data), dimnames = NULL)
     vl <- prod(dim)
     if( length(data) != vl  ) {
 	t1 <- ceiling(vl/length(data))
-	data <- rep(data,t1)
+	data <- rep.int(data,t1)
 	if( length(data) != vl )
 	    data <- data[1:vl]
     }
@@ -28,9 +28,9 @@ function(x, MARGIN)
         stop("incorrect value for MARGIN")
 
     if(any(d == 0)) return(array(integer(0), d))
-    
-    y <- rep(rep(seq(1 : d[MARGIN]),
-                 prod(d[seq(length = MARGIN - 1)]) * rep(1, d[MARGIN])),
+
+    y <- rep.int(rep.int(seq(1 : d[MARGIN]),
+                 prod(d[seq(length = MARGIN - 1)]) * rep.int(1, d[MARGIN])),
              prod(d[seq(from = MARGIN + 1, length = n - MARGIN)]))
     dim(y) <- d
     y
