@@ -1,4 +1,4 @@
 ## gl function of GLIM
 gl <- function (n, k, length = n*k, labels=1:n, ordered=FALSE)
-    factor(rep(rep(1:n,rep(k,n)), length=length),
+    factor(rep(rep.int(1:n, rep.int(k,n)), length=length),
 	   levels=1:n, labels=labels, ordered=ordered)
