@@ -1,7 +1,7 @@
 
 addTclPath <- function(path = ".")
 {
-    a <- .Tcl("set auto_path")
+    a <- tclvalue(.Tcl("set auto_path"))
     paths <- strsplit(a," ")[[1]]
     if (is.na(match(path, paths)))
         .Tcl(paste("lappend auto_path ", path))
@@ -10,7 +10,7 @@ addTclPath <- function(path = ".")
 
 tclRequire <- function(package, warn = TRUE)
 {
-    a <- .Tcl(paste("package versions ", package))
+    a <- tclvalue(.Tcl(paste("package versions ", package)))
     if (length(a)==1 && nchar(a)==""){
         if (warn) warning(paste("Tcl package", package, "not found."))
         return(FALSE)
