@@ -1,4 +1,4 @@
-.First.lib <- function(lib, pkg)
+.onLoad <- function(lib, pkg)
 {
     if(nchar(Sys.getenv("MY_TCLTK"))) {
         library.dynam("tcltk", pkg, lib)
@@ -19,14 +19,11 @@
     invisible(addTclPath(extra))
 }
 
-.Last.lib <- function(libpath) {
+.onUnload <- function(libpath) {
     if(is.loaded(symbol.C("tcltk_end"))) {
         .C("tcltk_end", PACKAGE="tcltk")
 ## unloading the DLL used to work, but it seems Tcl/Tk 8.4.1 does
 ## not like being reinitialized
-#         dyn.unload(file.path(libpath, "libs", "tcltk.dll"))
-#         .Dyn.libs <- .dynLibs()
-#         num <- match("tcltk", .Dyn.libs)
-#         .dynLibs(.Dyn.libs[-num])
+        ## library.dynam.unload("tcltk", libpath)
     }
 }
