@@ -18,7 +18,6 @@ arima.sim <- function(model, n, rand.gen = rnorm,
                       innov = rand.gen(n, ...), n.start = NA, ...)
 {
     if(!is.list(model)) stop("`model' must be list")
-    nm <- names(model)
     p <- length(model$ar)
     if(p) {
         minroots <- min(Mod(polyroot(c(1, -model$ar))))
@@ -31,6 +30,8 @@ arima.sim <- function(model, n, rand.gen = rnorm,
     d <- 0
     if(!is.null(ord <- model$order)) {
         if(length(ord) != 3) stop("`model$order' must be of length 3")
+        if(p != ord[1]) stop("inconsistent specification of ar order")
+        if(q != ord[3]) stop("inconsistent specification of ma order")
         d <- ord[2]
         if(d != round(d) || d < 0)
             stop("number of differences must be a positive integer")
@@ -40,5 +41,5 @@ arima.sim <- function(model, n, rand.gen = rnorm,
     if(length(model$ar)) x <- filter(x, model$ar, method = "recursive")
     x <- x[-(1:n.start)]
     if(d > 0) x <- diffinv(x, differences = d)
-    else as.ts(x)
+    as.ts(x)
 }
