@@ -1,7 +1,9 @@
 .onLoad <- function(lib, pkg)
 {
-#    library.dynam("ts", pkg, lib)
     options(ts.S.compat = FALSE)
 }
 
 .noGenerics <- TRUE
+
+.onUnload <- function(libpath)
+    library.dynam.unload("ts", libpath)
