@@ -2,6 +2,7 @@
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996, 1997  Robert Gentleman and Ross Ihaka
  *  Copyright (C) 1998--2003	    The R Development Core Team.
+ *  Copyright (C) 2003		    The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +14,10 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  A copy of the GNU General Public License is available via WWW at
+ *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
+ *  writing to the Free Software Foundation, Inc., 59 Temple Place,
+ *  Suite 330, Boston, MA  02111-1307  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1068,11 +1070,20 @@ static SEXP math2(SEXP sa, SEXP sb, double (*f)(), SEXP lcall)
     if (!isNumeric(sa) || !isNumeric(sb))
 	errorcall(lcall, R_MSG_NONNUM_MATH);
 
+    /* for 0-length a we want the attributes of a, not those of b
+       as no recycling will occur */
 #define SETUP_Math2				\
     na = LENGTH(sa);				\
     nb = LENGTH(sb);				\
-    if ((na == 0) || (nb == 0))			\
-	return(allocVector(REALSXP, 0));	\
+    if ((na == 0) || (nb == 0))	{		\
+        PROTECT(sy = allocVector(REALSXP, 0));	\
+        if (na == 0) {				\
+	    SET_ATTRIB(sy, duplicate(ATTRIB(sa)));\
+	    SET_OBJECT(sy, sao);		\
+        }					\
+        UNPROTECT(1);				\
+	return(sy);				\
+    }						\
     n = (na < nb) ? nb : na;			\
     PROTECT(sa = coerceVector(sa, REALSXP));	\
     PROTECT(sb = coerceVector(sb, REALSXP));	\
