@@ -417,7 +417,10 @@ static void symcrossprod(double *x, int nr, int nc, double *z)
         F77_CALL(dsyrk)(uplo, trans, &nc, &nr, &one, x, &nr, &zero, z, &nc);
 	for (i = 1; i < nc; i++)
 	    for (j = 0; j < i; j++) z[i + nc *j] = z[j + nc * i];
+    } else { /* zero-extent operations should return zeroes */
+	for(i = 0; i < nc*nc; i++) z[i] = 0;
     }
+
 }
 
 static void crossprod(double *x, int nrx, int ncx,
@@ -430,6 +433,10 @@ static void crossprod(double *x, int nrx, int ncx,
         F77_CALL(dgemm)(transa, transb, &ncx, &ncy, &nrx, &one,
 			x, &nrx, y, &nry, &zero, z, &ncx);
     }
+    else { /* zero-extent operations should return zeroes */
+	int i;
+	for(i = 0; i < ncx*ncy; i++) z[i] = 0;
+    }
 #else
     int i, j, k;
     double xji, yjk, sum;
@@ -464,6 +471,10 @@ static void ccrossprod(Rcomplex *x, int nrx, int ncx,
         F77_CALL(zgemm)(transa, transb, &ncx, &ncy, &nrx, &one,
 			x, &nrx, y, &nry, &zero, z, &ncx);
     }
+    else { /* zero-extent operations should return zeroes */
+	int i;
+	for(i = 0; i < ncx*ncy; i++) z[i].r = z[i].i = 0;
+    }
 #else
     int i, j, k;
     double xji_r, xji_i, yjk_r, yjk_i, sum_r, sum_i;
