@@ -191,9 +191,18 @@ SEXP do_relop_dflt(SEXP call, SEXP op, SEXP x, SEXP y)
     return x;
 }
 
+/* i1 = i % n1; i2 = i % n2;
+ * this macro is quite a bit faster than having real modulo calls
+ * in the loop (tested on Intel and Sparc)
+ */
+#define mod_iterate(n1,n2,i1,i2) for (i=i1=i2=0; i<n; \
+	i1 = (++i1 == n1) ? 0 : i1,\
+	i2 = (++i2 == n2) ? 0 : i2,\
+	++i)
+
 static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 {
-    int i, n, n1, n2;
+    int i, i1, i2, n, n1, n2;
     int x1, x2;
     SEXP ans;
 
@@ -206,9 +215,9 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 
     switch (code) {
     case EQOP:
-	for (i = 0; i < n; i++) {
-	    x1 = INTEGER(s1)[i % n1];
-	    x2 = INTEGER(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = INTEGER(s1)[i1];
+	    x2 = INTEGER(s2)[i2];
 	    if (x1 == NA_INTEGER || x2 == NA_INTEGER)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -216,9 +225,9 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case NEOP:
-	for (i = 0; i < n; i++) {
-	    x1 = INTEGER(s1)[i % n1];
-	    x2 = INTEGER(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = INTEGER(s1)[i1];
+	    x2 = INTEGER(s2)[i2];
 	    if (x1 == NA_INTEGER || x2 == NA_INTEGER)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -226,9 +235,9 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case LTOP:
-	for (i = 0; i < n; i++) {
-	    x1 = INTEGER(s1)[i % n1];
-	    x2 = INTEGER(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = INTEGER(s1)[i1];
+	    x2 = INTEGER(s2)[i2];
 	    if (x1 == NA_INTEGER || x2 == NA_INTEGER)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -236,9 +245,9 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case GTOP:
-	for (i = 0; i < n; i++) {
-	    x1 = INTEGER(s1)[i % n1];
-	    x2 = INTEGER(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = INTEGER(s1)[i1];
+	    x2 = INTEGER(s2)[i2];
 	    if (x1 == NA_INTEGER || x2 == NA_INTEGER)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -246,9 +255,9 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case LEOP:
-	for (i = 0; i < n; i++) {
-	    x1 = INTEGER(s1)[i % n1];
-	    x2 = INTEGER(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = INTEGER(s1)[i1];
+	    x2 = INTEGER(s2)[i2];
 	    if (x1 == NA_INTEGER || x2 == NA_INTEGER)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -256,9 +265,9 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case GEOP:
-	for (i = 0; i < n; i++) {
-	    x1 = INTEGER(s1)[i % n1];
-	    x2 = INTEGER(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = INTEGER(s1)[i1];
+	    x2 = INTEGER(s2)[i2];
 	    if (x1 == NA_INTEGER || x2 == NA_INTEGER)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -272,7 +281,7 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 
 static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 {
-    int i, n, n1, n2;
+    int i, i1, i2, n, n1, n2;
     double x1, x2;
     SEXP ans;
 
@@ -285,9 +294,9 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 
     switch (code) {
     case EQOP:
-	for (i = 0; i < n; i++) {
-	    x1 = REAL(s1)[i % n1];
-	    x2 = REAL(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = REAL(s1)[i1];
+	    x2 = REAL(s2)[i2];
 	    if (ISNAN(x1) || ISNAN(x2))
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -295,9 +304,9 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case NEOP:
-	for (i = 0; i < n; i++) {
-	    x1 = REAL(s1)[i % n1];
-	    x2 = REAL(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = REAL(s1)[i1];
+	    x2 = REAL(s2)[i2];
 	    if (ISNAN(x1) || ISNAN(x2))
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -305,9 +314,9 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case LTOP:
-	for (i = 0; i < n; i++) {
-	    x1 = REAL(s1)[i % n1];
-	    x2 = REAL(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = REAL(s1)[i1];
+	    x2 = REAL(s2)[i2];
 	    if (ISNAN(x1) || ISNAN(x2))
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -315,9 +324,9 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case GTOP:
-	for (i = 0; i < n; i++) {
-	    x1 = REAL(s1)[i % n1];
-	    x2 = REAL(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = REAL(s1)[i1];
+	    x2 = REAL(s2)[i2];
 	    if (ISNAN(x1) || ISNAN(x2))
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -325,9 +334,9 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case LEOP:
-	for (i = 0; i < n; i++) {
-	    x1 = REAL(s1)[i % n1];
-	    x2 = REAL(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = REAL(s1)[i1];
+	    x2 = REAL(s2)[i2];
 	    if (ISNAN(x1) || ISNAN(x2))
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -335,9 +344,9 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	}
 	break;
     case GEOP:
-	for (i = 0; i < n; i++) {
-	    x1 = REAL(s1)[i % n1];
-	    x2 = REAL(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = REAL(s1)[i1];
+	    x2 = REAL(s2)[i2];
 	    if (ISNAN(x1) || ISNAN(x2))
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
@@ -351,7 +360,7 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 
 static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
 {
-    int i, n, n1, n2;
+    int i, i1, i2, n, n1, n2;
     Rcomplex x1, x2;
     SEXP ans;
 
@@ -368,9 +377,9 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
 
     switch (code) {
     case EQOP:
-	for (i = 0; i < n; i++) {
-	    x1 = COMPLEX(s1)[i % n1];
-	    x2 = COMPLEX(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = COMPLEX(s1)[i1];
+	    x2 = COMPLEX(s2)[i2];
 	    if (ISNAN(x1.r) || ISNAN(x1.i) ||
 		ISNAN(x2.r) || ISNAN(x2.i))
 		LOGICAL(ans)[i] = NA_LOGICAL;
@@ -379,9 +388,9 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
 	}
 	break;
     case NEOP:
-	for (i = 0; i < n; i++) {
-	    x1 = COMPLEX(s1)[i % n1];
-	    x2 = COMPLEX(s2)[i % n2];
+	mod_iterate(n1, n2, i1, i2) {
+	    x1 = COMPLEX(s1)[i1];
+	    x2 = COMPLEX(s2)[i2];
 	    if (ISNAN(x1.r) || ISNAN(x1.i) ||
 		ISNAN(x2.r) || ISNAN(x2.i))
 		LOGICAL(ans)[i] = NA_LOGICAL;
@@ -402,9 +411,6 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
 #define STRCMP strcmp
 #endif
 
-/* define NASTRING to have NA_STRING treated as missing */
-#define NASTRING
-
 static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 {
     int i, n, n1, n2;
@@ -420,12 +426,10 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
     switch (code) {
     case EQOP:
 	for (i = 0; i < n; i++) {
-#ifdef NASTRING
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
  	    else
-#endif
 	    if (strcmp(CHAR(STRING_ELT(s1, i % n1)),
 		       CHAR(STRING_ELT(s2, i % n2))) == 0)
 		LOGICAL(ans)[i] = 1;
@@ -435,12 +439,10 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	break;
     case NEOP:
 	for (i = 0; i < n; i++) {
-#ifdef NASTRING
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
  	    else
-#endif
 	    if (streql(CHAR(STRING_ELT(s1, i % n1)),
 		       CHAR(STRING_ELT(s2, i % n2))) != 0)
 		LOGICAL(ans)[i] = 0;
@@ -450,12 +452,10 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	break;
     case LTOP:
 	for (i = 0; i < n; i++) {
-#ifdef NASTRING
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
  	    else
-#endif
 	    if (STRCMP(CHAR(STRING_ELT(s1, i % n1)),
 		       CHAR(STRING_ELT(s2, i % n2))) < 0)
 		LOGICAL(ans)[i] = 1;
@@ -465,12 +465,10 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	break;
     case GTOP:
 	for (i = 0; i < n; i++) {
-#ifdef NASTRING
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
  	    else
-#endif
 	    if (STRCMP(CHAR(STRING_ELT(s1, i % n1)),
 		       CHAR(STRING_ELT(s2, i % n2))) > 0)
 		LOGICAL(ans)[i] = 1;
@@ -480,12 +478,10 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	break;
     case LEOP:
 	for (i = 0; i < n; i++) {
-#ifdef NASTRING
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
  	    else
-#endif
 	    if (STRCMP(CHAR(STRING_ELT(s1, i % n1)),
 		       CHAR(STRING_ELT(s2, i % n2))) <= 0)
 		LOGICAL(ans)[i] = 1;
@@ -495,12 +491,10 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	break;
     case GEOP:
 	for (i = 0; i < n; i++) {
-#ifdef NASTRING
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
  	    else
-#endif
 	    if (STRCMP(CHAR(STRING_ELT(s1, i % n1)),
 		       CHAR(STRING_ELT(s2, i % n2))) >= 0)
 		LOGICAL(ans)[i] = 1;
