@@ -116,8 +116,8 @@ e		9			0
 
 */
 
-int		NumOfRoots = 0;
-int		*RootItems = NULL; 
+UInt32		NumOfRoots = 0;
+DataBrowserItemID	*RootItems = NULL; 
 int		**SubItemsID;
 int		CurrentID = 0;
 void	InitContainers(void);
@@ -141,7 +141,7 @@ Rect WSBounds = { 400, 400, 600, 800 };
 int *IDNum;              /* id          */
 Rboolean *IsRoot;        /* isroot      */
 Rboolean *IsContainer;   /* iscontainer */
-int *NumberOfItems;      /* numofit     */
+UInt32 *NumberOfItems;      /* numofit     */
 int *ParentID;           /* parid       */
 char **Names;            /* name        */
 char **Types;            /* type        */
@@ -228,8 +228,8 @@ SEXP Raqua_do_wsbrowser(SEXP call, SEXP op, SEXP args, SEXP env)
  
   IDNum = (int*)malloc(NumOfID * sizeof(int));
   IsRoot = (Rboolean*)malloc(NumOfID * sizeof(Rboolean));
-  IsContainer = (Rboolean*)malloc(NumOfID * sizeof(Rboolean));
-  NumberOfItems = (int*)malloc(NumOfID * sizeof(int));
+  IsContainer = (Rboolean *)malloc(NumOfID * sizeof(Rboolean));
+  NumberOfItems = (UInt32 *)malloc(NumOfID * sizeof(UInt32));
   ParentID = (int*)malloc(NumOfID * sizeof(int));
    
   for(i=0; i<NumOfID; i++){
@@ -249,11 +249,15 @@ SEXP Raqua_do_wsbrowser(SEXP call, SEXP op, SEXP args, SEXP env)
 	else
 	    Sizes[i] = CHAR(R_BlankString);  
 
-	IDNum[i] = INTEGER(id)[i];
+    IDNum[i] = INTEGER(id)[i];
     NumberOfItems[i] = INTEGER(numofit)[i];
+    if(INTEGER(parid)[i] == -1)
+     ParentID[i] = kDataBrowserNoItem;
+    else 
     ParentID[i] = INTEGER(parid)[i]; 
     IsRoot[i] = LOGICAL(isroot)[i];
     IsContainer[i] = LOGICAL(iscont)[i];
+   
   }
 
   OpenDataBrowser();
@@ -274,7 +278,7 @@ void InitContainers(void){
 	  NumOfRoots++;
 	if(RootItems) free(RootItems);
 	  
-	RootItems = malloc(sizeof(int)*NumOfRoots);
+	RootItems = malloc(sizeof(DataBrowserItemID)*NumOfRoots);
 	SubItemsID = malloc(NumOfID * sizeof(int*));
 
 	for(i = 0; i < NumOfID; i++){
@@ -288,7 +292,10 @@ void InitContainers(void){
           l++;
          }
        }
-     }   /* if(IsContainer[i]) */
+     } else {   /* if(IsContainer[i]) */
+      SubItemsID[i] = malloc(sizeof(int));
+      SubItemsID[i][0] = 0;
+     }
 	} /* for */
 
 }
@@ -342,11 +349,10 @@ void OpenDataBrowser(void)
     /* Configure the DataBrowser */
 	if(!isBrowserOpen)
 	 ConfigureDataBrowser(WSpaceBrowser);
-    err = SetDataBrowserTarget(WSpaceBrowser, 1);
+    err = SetDataBrowserTarget(WSpaceBrowser, 0);
    
     AddDataBrowserItems(WSpaceBrowser, kDataBrowserNoItem, 
-				(UInt32)NumOfRoots, (DataBrowserItemID *)RootItems, kDataBrowserItemNoProperty);
- 
+				NumOfRoots, RootItems, kDataBrowserItemNoProperty);
     /* Set the keyboard focus */
 	SetKeyboardFocus(BrowserWindow, WSpaceBrowser, kControlDataBrowserPart);
 	
@@ -618,7 +624,7 @@ static pascal OSStatus MyGetSetItemData(ControlRef browser,
 		case kObjectColumn:
 		{
 			CFStringRef text;
-			CopyCStringToPascal(Names[itemID-1],pascalString);
+                        CopyCStringToPascal(Names[itemID-1],pascalString);
 			text = CFStringCreateWithPascalString(
 				CFAllocatorGetDefault(), pascalString, kCFStringEncodingMacRoman);
 			err = SetDataBrowserItemDataText(itemData, text); 
@@ -692,7 +698,7 @@ static pascal OSStatus MyGetSetItemData(ControlRef browser,
 		
 		case kDataBrowserContainerOpened:
 		{	
-			AddDataBrowserItems(browser, itemID, (UInt32)NumberOfItems[itemID-1], (DataBrowserItemID *)SubItemsID[itemID-1], kObjectColumn);
+                AddDataBrowserItems(browser, itemID, NumberOfItems[itemID-1], (DataBrowserItemID *)SubItemsID[itemID-1], kObjectColumn);
 			{	
 				Boolean variableHeightRows;
 				GetDataBrowserTableViewGeometry(
