@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998 Ross Ihaka
- *  Copyright (C) 2000, 2002 The R Development Core Team
+ *  Copyright (C) 2000, 2003 The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,11 +40,10 @@
 #include "nmath.h"
 
 #ifndef MATHLIB_STANDALONE
-/* back-compatibility entry point */
-double Rf_log1p(double x)
-{
-    return log1p(x);
-}
+/* want to compile log1p as Rlog1p if HAVE_LOG1P && !HAVE_WORKING_LOG1P */
+#if defined(HAVE_LOG1P) && !defined(HAVE_WORKING_LOG1P)
+#undef HAVE_LOG1P
+#endif
 #endif
 
 #ifndef HAVE_LOG1P
