@@ -719,9 +719,12 @@ stopifnot(abs(X - s$u %*% D %*% t(s$v)) < Eps)#	 X = U D V'
 stopifnot(abs(D - t(s$u) %*% X %*% s$v) < Eps)#	 D = U' X V
 ## end of moved from svd.Rd
 
+hasMethods <- .isMethodsDispatchOn() ## (for setting back)
+## was at end, as package `methods' once had persistent side effects
+stopifnot(require(methods))
+stopifnot(all.equal(3:3, 3.), all.equal(1., 1:1))
 
-## trace
-hasMethods <- .isMethodsDispatchOn() ## trace requires methods
+## trace (requiring methods):
 f <- function(x, y) { c(x,y)}
 xy <- 0
 
@@ -2654,6 +2657,7 @@ stopifnot(identical(TRUE, attr(as.dist(m, diag=TRUE), "Diag")))
 stopifnot(1:2 == ave(1:2,factor(2:3,levels=1:3)))
 ## gave "2 NA" previous to 1.8.0, because unused levels weren't dropped
 
+
 ## PR#4092: arrays with length(dim(.)) = 1
 z <- array(c(-2:1, 1.4),5)
 cz <- crossprod(as.vector(z))
@@ -2665,10 +2669,12 @@ stopifnot(crossprod(z) == cz,# the first has NULL dimnames
           identical(crossprod(z), crossprod(z,z0)))
 ## crossprod(z) segfaulted (or gave silly error message) before 1.8.0
 
+
 ## PR#4431
 stopifnot(!is.na(rmultinom(12,100, c(3, 4, 2, 0,0))))
 ## 3rd line was all NA before 1.8.0
 
+
 ## PR#4275: getAnywhere with extra "."
 g0 <- getAnywhere("predict.loess")
 g1 <- getAnywhere("as.dendrogram.hclust")
@@ -2679,7 +2685,92 @@ stopifnot(is.S3meth(g0), is.S3meth(g1),
           is.S3meth(g2), is.S3meth(g3))
 ## all but g0 failed until 1.8.0 (Oct 6)
 
-## keep at end, as package `methods' has had persistent side effects
-library(methods)
-stopifnot(all.equal(3:3, 3.), all.equal(1., 1:1))
-detach("package:methods")
+
+## symnum(x) for length 0 and some logical arrays:
+sm <- symnum(m <- matrix(1:8 %% 3 == 0, 2))
+stopifnot(identical(symnum(FALSE[FALSE]), noquote(""[FALSE])),
+          identical(symnum(c(m)), c(symnum(m))),
+          dim(sm) == dim(m), class(sm) == "noquote")
+## symnum(<length 0>) gave noquote("()") before 1.8.1
+
+
+## abbreviate with leading (or trailing) space differences (PR#4564)
+abbreviate(c("A"," A"), 4)
+## this gave infinite loop before 1.8.1
+
+
+## crossprod on 0-extent matrices
+a <- matrix(,0,5)
+stopifnot(crossprod(a) == 0)
+stopifnot(crossprod(a,a) == 0)
+stopifnot(crossprod(a+0i) == 0+0i)
+## were random areas in <= 1.8.0
+
+
+## DF[[i, j]] should be row i, col j
+data(women)
+stopifnot(women[[2, 1]] == women[2, 1])
+women[[2, 1]] <- 77
+stopifnot(women[2, 1] == 77)
+## was reversed from May 2002 to Oct 2003
+
+
+## merge.data.frame with a single-column df (PR#4299)
+x <- data.frame(x = 1:5, y = letters[1:5])
+y <- data.frame(z = 1:2)
+z <- merge(x, y)
+stopifnot(identical(names(z), c("x", "y", "z")))
+## third name was wrong in 1.8.0
+
+
+## cor(mat, use = "pair") was plainly wrong
+data(longley) # has no NA's -- hence all "use = " should give the same!
+X <- longley
+ep <- 32 * .Machine$double.eps
+for(meth in eval(formals(cor)$method)) {
+    cat("method = ", meth,"\n")
+    Cl <- cor(X, method = meth)
+    stopifnot(all.equal(Cl, cor(X, method= meth, use= "complete"), tol=ep),
+              all.equal(Cl, cor(X, method= meth, use= "pairwise"), tol=ep),
+              all.equal(Cl, cor(X, X, method= meth), tol=ep),
+              all.equal(Cl, cor(X, X, method= meth, use= "pairwise"), tol=ep),
+              all.equal(Cl, cor(X, X, method= meth, use= "pairwise"), tol=ep)
+              )
+}
+## "pairwise" failed in 1.8.0
+
+
+## regexpr(*, fixed=TRUE) had 0-index from C
+txt <- c("english", "french", "swiss")
+ir <- regexpr("en", txt, fixed = TRUE)
+stopifnot(ir == c(1, 3, -1),
+          identical(ir, regexpr("en", txt)))
+## (*, fixed=TRUE) gave 0 2 -1 before R 1.8.1
+
+
+##-- S4 classes with S3 slots:
+setClass("test1", representation(date="POSIXct"))
+(x <- new("test1", date=as.POSIXct("2003-10-09")))
+stopifnot(format(x @ date) == "2003-10-09")
+## line 2 failed in 1.8.0 because of an extraneous space in "%in%"
+
+
+## PR#5017: filter(init=) had the wrong time order
+xx <- filter(4:8, c(1, 0.5, 0.25), method="recursive", init=3:1)
+stopifnot(identical(xx[1:3], c(8.25, 15.25, 26.125)))
+## 1.8.0 gave 6.75 12.75 22.375
+
+
+## PR#5090 user error with writeChar could segfault
+tf <- tempfile()
+zz <- file(tf, "wb")
+writeChar("", zz, nchars=10000000)
+close(zz)
+unlink(tf)
+## segfaults in 1.8.0
+
+
+## PR#4710 round (and signif) dropped attributes
+x <- round(matrix(0, 0, 3))
+stopifnot(identical(dim(x), as.integer(c(0, 3))))
+## numeric(0) in 1.8.0
