@@ -1,4 +1,4 @@
-### $Id: pkg2tex.pl,v 1.3 2002/11/03 17:32:29 hornik Exp $
+### $Id: pkg2tex.pl,v 1.4 2003/07/29 10:35:13 ripley Exp $
 
 ## Create a single pkgname-pkg.tex file from the Latex subdirectories
 ## Copyright (C) 1998 Douglas M. Bates <bates@stat.wisc.edu>
@@ -28,10 +28,11 @@ use strict;
 use FileHandle;
 use Carp;
 use Getopt::Long;
+use R::Utils;
 
 my $help;
 
-my $revision = ' $Revision: 1.3 $ ';
+my $revision = ' $Revision: 1.4 $ ';
 my $version;
 my $name;
 
@@ -55,12 +56,22 @@ for (@ARGV) {
     my $latexDir = $RLIB . "/" . $_ . "/latex/";
     carp "latex subdirectory for library $_ does not exist!\n", next
 	unless -d $latexDir;
+    my $was_zipped = 0;
+    if(-f $latexDir . "Rhelp.zip") {
+	$was_zipped = 1;
+	my $cmd = "cd $latexDir; unzip -qo Rhelp.zip";
+	croak "Cannot unzip latex files\n" if R_system($cmd);
+    }
     my $out = new FileHandle "> " . $_ . "-pkg.tex" or
 	croak "unable to open file $_-pkg.tex: $!\n";
     &do_header($_, $out);
     &do_tex_files($latexDir, $out);
     &do_trailer($out);
     $out->close;
+    if($was_zipped) {
+	croak "Removing unzipped latex files failed.\n" if
+	    R_system("rm -f $latexDir*.tex");
+    }
 }
 
 sub do_header {
